"""
Módulo para definição de objetos
que serão obtidos na página de Gerenciamento de Perfis
Cadastro
"""

from typing import Literal, Optional

from pydantic import BaseModel, EmailStr, field_validator


class Cadastro(BaseModel):
    """
    Objeto "Cadastro"
    """

    nome: str
    nome_social: Optional[str] = None
    email: EmailStr
    email_alternativo: Optional[EmailStr] = None
    cpf: str
    rg: Optional[str] = None
    rg_orgao_emissor: Optional[str] = None
    telefone: Optional[str] = None
    celular: Optional[str] = None
    genero: Literal['Masculino', 'Feminino']

    @field_validator('*')
    def empty_str_to_none(cls, v):
        """
        Se o valor for '', converte para None
        https://github.com/pydantic/pydantic/discussions/2687
        """
        if v == '':
            return None
        return v


class Perfil(BaseModel):
    """
    Objeto "Perfil"
    """

    perfil: str
    autorizado: bool
    descricao: str
    observacao: Optional[str] = None


if __name__ == '__main__':
    perfil = Perfil(
        perfil='Distribuidor Intimações MP - Primeira Instância',
        autorizado=True,
        descricao='Este perfil permite que você tenha acesso aos serviços restritos do Portal e-SAJ, como Intimações Online e Consulta de Processos',
        observacao='Locais de atuação:\nTodos os Foros e Varas',
    )
    print(perfil)
    print(perfil.perfil)

    cad = Cadastro(
        nome='Michel Metran',
        genero='Masculino',
        email='michel@mail.com.br',
        cpf='meu cpf',
        rg='',
    )
    print(cad.email)



"""
Módulo com a página de Consulta de Processos
de 1º e 2º graus

Não precisa, obrigatoriamente, estar logado.
Mas garante melhores permissões.

Os filtros de processos são os mesmos para 1º e 2º graus
Talvez o conteúdo seja diferente.
"""

import warnings
from typing import List, Literal

from selenium.webdriver.common.by import By

from pyesaj.scraper.pom import Page, PageElement


# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.common.action_chains import ActionChains


# flake8: noqa:303
# flake8: noqa:E501


class ConsultaProcesso(Page):
    """
    Páginas:
    "Consultas Processuais > Consulta de Processos do 1º Grau"\n
    "Consultas Processuais > Consulta de Processos do 2º Grau"
    """

    url_1grau = 'https://esaj.tjsp.jus.br/cpopg/open.do'
    url_2grau = 'https://esaj.tjsp.jus.br/cposg/open.do'

    def __init__(
        self, driver, grau: Literal['Primeiro Grau', 'Segundo Grau']
    ) -> None:
        super().__init__(driver)

        if grau == 'Primeiro Grau':
            self.go_to(self.url_1grau)

        elif grau == 'Segundo Grau':
            self.go_to(self.url_2grau)

        else:
            raise ValueError("Precisa ser 'Primeiro Grau' ou 'Segundo Grau'")


class ConsultaPor(PageElement):
    """
    Representa o menu *dropdown* "Consulta Por", que define
    qual o tipo de consulta processual que será feita.
    """

    selector_consulta_por = (
        By.XPATH,
        '//form[@name="consultarProcessoForm"]//select[@name="cbPesquisa"]',
    )

    def get_options(self) -> List[str]:
        """
        Obtem opções de preenchimento.
        """

        return self.create_select_options(self.selector_consulta_por)

    def set_option(self, option: str) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        list_options = self.get_options()
        if option not in list_options:
            raise ValueError(f'Precisa ser uma opção {list_options}')

        self.select(self.selector_consulta_por, option)


class NumeroUnificado(PageElement):
    """
    Representa o campo de *input* "Número do Processo",
    no padrão do TJSP.

    Só é possível pesquisar pelo Número do Processo Unificado,
    que contem o ".8.26." ou ".8-26." no meio do número.
    """

    numero_parte1 = (
        By.XPATH,
        '//div[@id="NUMPROC"]//input[@id="numeroDigitoAnoUnificado"]',
    )
    numero_parte3 = (
        By.XPATH,
        '//div[@id="NUMPROC"]//input[@id="foroNumeroUnificado"]',
    )

    def define_numero(self, numero: str) -> None:
        """
        Preenche o Número do Processo Unificado.
        """
        if '.8.26.' in numero:
            numero_split = str(numero).split('.8.26.')

        elif '.8-26.' in numero:
            numero_split = str(numero).split('.8-26.')

        else:
            raise ValueError(
                'O número não tem a sequencia ".8.26." para dividir o número em partes'
            )

        self.set(locator=self.numero_parte1, value=numero_split[0])
        self.set(locator=self.numero_parte3, value=numero_split[1])


class NomeParte(PageElement):
    """
    Representa o campo de *input* "Nome da Parte".
    """

    nome_parte = (By.XPATH, '//input[@id="campo_NMPARTE"]')
    check_box_nome_completo = (
        By.XPATH,
        #'//div[@id="NMPARTE"]//span[@id="pesquisarPorNomeCompleto"]',
        # '//div[@id="NMPARTE"]//span[contains(text(),"Pesquisar por nome completo")]',
        '//div[@id="NMPARTE"]//label',
    )

    def set_nome_parte(self, nome: str) -> None:
        """
        Define o "Nome da parte"
        """
        self.set(locator=self.nome_parte, value=nome)

    def pesquisa_nome_completo(self) -> None:
        """
        Representa o *checkbox* "Pesquisar por nome completo".
        """
        # TODO: Adicionar validação
        self.click(locator=self.check_box_nome_completo)


class DocumentoParte(PageElement):
    """
    Representa o campo de *input* "Documento da Parte".
    """

    nome_parte = (By.XPATH, '//input[@id="campo_DOCPARTE"]')

    def set_documento(self, documento: str) -> None:
        """
        Define "Documento da Parte".
        """
        self.set(locator=self.nome_parte, value=documento)


class NomeAdvogado(PageElement):
    """
    Representa do campo de *input* "Nome do Advogado".
    """

    nome_adv = (By.XPATH, '//input[@id="campo_NMADVOGADO"]')
    check_box_nome_completo = (
        By.XPATH,
        #'//div[@id="NMADVOGADO"]//span[@id="pesquisarPorNomeCompleto"]',
        '//div[@id="NMADVOGADO"]//label',
    )

    def set_nome_adv(self, nome: str) -> None:
        """
        Preenche o campo de *input* "Nome do Advogado".
        """
        self.set(locator=self.nome_adv, value=nome)

    def pesquisa_nome_completo(self) -> None:
        """
        Habilita o *checkbox* "Pesquisar por nome completo".
        """
        # TODO: Ideal fazer uma validação
        self.click(locator=self.check_box_nome_completo)


class OAB(PageElement):
    """
    Representa do campo de *input* "OAB".
    """

    oab = (By.XPATH, '//input[@id="campo_NUMOAB"]')

    def set_documento(self, documento: str) -> None:
        """
        Define a "OAB".
        """
        self.set(locator=self.oab, value=documento)


class NumCartaPrecatoria(PageElement):
    """
    Representa do campo de *input* "Nº da Carta Precatória na Origem".
    """

    nome_parte = (By.XPATH, '//input[@id="campo_PRECATORIA"]')

    def set_documento(self, documento: str) -> None:
        """
        Define o "Nº da Carta Precatória na Origem".
        """
        self.set(locator=self.nome_parte, value=documento)


class NumDocDelegacia(PageElement):
    """
    Representa do campo de *input* "Nº do Documento da Delegacia".
    """

    nome_parte = (By.XPATH, '//input[@id="campo_DOCDELEG"]')

    def set_documento(self, documento: str) -> None:
        """
        Define o "Nº do Documento da Delegacia".
        """
        self.set(locator=self.nome_parte, value=documento)


class CDA(PageElement):
    """
    Representa do campo de *input* "CDA" (Certidão da Dívida Ativa?!).
    """

    nome_parte = (By.XPATH, '//input[@id="campo_NUMCDA"]')

    def set_documento(self, documento: str) -> None:
        """
        Define o "CDA" (Certidão da Dívida Ativa?!).
        """
        self.set(locator=self.nome_parte, value=documento)


class Consultar(PageElement):
    """
    Representa o botão "Consultar"
    """

    btn_consultar_1grau = (
        By.XPATH,
        '//input[@id="botaoConsultarProcessos"]',
    )

    btn_consultar_2grau = (
        By.XPATH,
        '//input[@id="pbConsultar"]',
    )

    def consultar(self, grau: Literal['Primeiro Grau', 'Segundo Grau']) -> None:
        """
        Aperta o botão "Consultar"
        """
        if grau == 'Primeiro Grau':
            self.click(locator=self.btn_consultar_1grau)

        elif grau == 'Segundo Grau':
            self.click(locator=self.btn_consultar_2grau)


class CheckResults(PageElement):
    """
    Avalia o resultado da pesquisa de processos

    """

    msg_results = (
        By.XPATH,
        '//*[contains(text(), "Não existem informações disponíveis para os parâmetros informados.")]',
    )

    def tem_resultado(self) -> bool:
        """
        Avalia o resultado da pesquisa

        - `True` significa que tem regstros
        - `False` significa que NÃO tem regstros
        """
        try:
            # Find Errors
            tags = self.find_all(locator=self.msg_results, wait=3)
            print(tags[0].text)

            # Se tem
            if len(tags) == 1 and (
                tags[0].text
                == 'Não existem informações disponíveis para os parâmetros informados.'
            ):
                # Mensagem
                warnings.warn(
                    message=f'\n'
                    f'A pesquisa não retornou nenhum resultado.\n'
                    f'\n',
                    category=RuntimeWarning,
                )
                # Se encontra a mensagem, significa que não tem resultado
                return False

            else:
                return False

        except:
            # Se não encontra a mensagem, significa que tem resultado!
            return True


if __name__ == '__main__':
    import os
    import time

    from dotenv import load_dotenv

    from pyesaj import webdriver

    # Credenciais
    load_dotenv()
    USERNAME = os.getenv('USERNAME_TJSP')
    PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver = webdriver.Firefox(
        # driver_path=paths.driver_path,
        # logs_path=paths.app_logs_path,
        # down_path=paths.driver_path,
        verify_ssl=False,
    )

    # Intimações
    consulta = ConsultaProcesso(driver=driver, grau='Primeiro Grau')
    # consulta.define_instancia(grau='1º grau')

    # Entra no eSAJ e loga
    # esaj = pyesaj.Login(driver=driver)
    # esaj.login(username=USERNAME, password=PASSWORD)

    # Consultar Por
    por = ConsultaPor(driver=driver)
    print(por.get_options())
    por.set_option(option='Número do Processo')

    # Número
    num = NumeroUnificado(driver=driver)
    num.define_numero(numero='0123479-07.2012.8.26.0100')

    # Nome da Parte
    por.set_option(option='Nome da parte')
    nom = NomeParte(driver=driver)
    nom.set_nome_parte(nome='Michel Metran')
    nom.pesquisa_nome_completo()

    # Documento da Parte
    por.set_option(option='Documento da Parte')
    doc = DocumentoParte(driver=driver)
    doc.set_documento(documento='324423324')

    # Nome do Advogado
    por.set_option(option='Nome do Advogado')
    adv = NomeAdvogado(driver=driver)
    adv.set_nome_adv(nome='Zé')
    adv.pesquisa_nome_completo()

    # Nome do Advogado
    por.set_option(option='OAB')
    o = OAB(driver=driver)
    o.set_documento(documento='324423324')

    # Nº da Carta Precatória na Origem
    por.set_option(option='Nº da Carta Precatória na Origem')
    a2 = NumCartaPrecatoria(driver=driver)
    a2.set_documento(documento='Cartaprecatoria')

    # Nº do Documento na Delegacia
    por.set_option(option='Nº do Documento na Delegacia')
    a3 = NumDocDelegacia(driver=driver)
    a3.set_documento(documento='Doc Delegacia')

    # CDA
    por.set_option(option='CDA')
    a4o = CDA(driver=driver)
    a4o.set_documento(documento='CDA')

    # Nome da Parte
    por.set_option(option='Nome da parte')
    nom = NomeParte(driver=driver)
    nom.set_nome_parte(nome='Michel Metran')
    nom.pesquisa_nome_completo()

    # Consultar
    con = Consultar(driver=driver)
    con.consultar()

    time.sleep(2)
    driver.quit()



"""
Módulo para componentes que estão em ambas páginas
Constam tanto em "Consulta de Intimações"
Constam também em "Recebimento de Intimações"
"""

import tempfile
import time
import logging
from pathlib import Path
from typing import List, Literal
from urllib.parse import urlparse

from selenium.webdriver.common.by import By

from pyesaj.scraper.params.intim.output import RecebimentoIntimacoes1Grau, \
    RecebimentoIntimacoes2Grau, ConsultaIntimacoes2Grau
from pyesaj.scraper.page.components.components import (
    InputModelSearchSet,
    InputModelTree,
)
from pyesaj.scraper.pom import PageElement

from pyesaj.scraper.webdriver import config
from pyesaj.scraper.outros.logger import disable_logging, \
    disable_logging_property


# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.common.action_chains import ActionChains


# flake8: noqa:303
# flake8: noqa:E501


class EmNomeDe(PageElement):
    """
    Representa o menu *dropdown* "Em nome de", que define
    qual o perfil do usuário que será utilizado.

    Pode ser pesquisa de intimações recebidas pelo nome do usuário,
    pessoa física, ou a pesquisa pelo
    "Ministério Público do Estado de São Paulo",
    caso o usuário seja funcionário dessa instituição.
    """

    selector_em_nome_de = (By.XPATH, '//select[@id="comboUsuarioReferencia"]')

    def get_options(self) -> List[str]:
        """
        Obtem opções de preenchimento.
        """
        return self.create_select_options(self.selector_em_nome_de)

    def set_option(self, option) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        list_options = self.get_options()
        if option not in list_options:
            raise ValueError(f'Precisa ser uma opção {list_options}')

        self.select(self.selector_em_nome_de, option)


class TipoParticipacao(PageElement):
    """
    Representa o campo de *input* "Tipo de Participação", que define
    com qual perfil o usuário fará a consulta.

    A opção desse campo depende da opção selecionada no campo "Em nome de".
    Caso tenha sido definido "Ministério Público do Estado de São Paulo",
    não há necessidade de definir o "Tipo de Participação".
    """

    selector_tipo_participacao = (
        By.XPATH,
        '//select[@id="comboTipoParticipacao"]',
    )

    def __init__(self, driver) -> None:
        super().__init__(driver)
        # self.selector_tipo_participacao =

    def get_options(self, options_delete=None) -> List[str]:
        """
        Obtem opções de preenchimento.
        """
        return self.create_select_options(
            self.selector_tipo_participacao, options_delete
        )

    def set_option(self, option) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        list_options = self.get_options()
        if len(list_options) == 0:
            msg = 'Não há campo "Tipo de Participação" para esse usuário!'
            print(msg)
            # logging.info(msg)

        elif option not in list_options:
            raise ValueError(f'Precisa ser uma opção {list_options}.')

        else:
            self.select(locator=self.selector_tipo_participacao, option=option)


class Instancia(PageElement):
    """
    Representa o *radio button* "Instância".
    """

    grau1_xpath = (By.XPATH, '//input[@id="primeiroGrau"]')
    grau2_xpath = (By.XPATH, '//input[@id="segundoGrau"]')
    grau3_xpath = (By.XPATH, '//input[@id="turmasRecursais"]')

    btn_consultar = (By.XPATH, '//input[@id="pbConsultar"]')
    label_foro_secao = (By.XPATH, '//td[@id="labelForo"]/label')

    def get_selected_option(
        self,
    ) -> (
        Literal['Primeiro Grau']
        | Literal['Segundo Grau']
        | Literal['Turmas Recursais']
    ):
        """
        Obtém instância selecionada.
        """
        # Pega Atributos
        check_1 = self.attribute(locator=self.grau1_xpath, attribute='checked')
        check_2 = self.attribute(locator=self.grau2_xpath, attribute='checked')
        check_3 = self.attribute(locator=self.grau3_xpath, attribute='checked')

        # Avalia quem está com atributo "checked"
        if check_1 is not None:
            return 'Primeiro Grau'

        elif check_2 is not None:
            return 'Segundo Grau'

        elif check_3 is not None:
            return 'Turmas Recursais'

        else:
            raise ValueError(
                'Aparentemente, não há nenhuma instância habilitada.'
            )

    def _get_xpath_instance(
        self,
        instancia: Literal['Primeiro Grau', 'Segundo Grau', 'Turmas Recursais'],
    ):
        """
        Determina qual o xPath de interesse
        Método Privado.
        """
        # instancia = self.get_selected_option()
        # print(f'Instância definida como: {instancia}')

        # Pega o xPath selecionado
        if instancia == 'Primeiro Grau':
            instancia_xpath = self.find(self.grau1_xpath)

        elif instancia == 'Segundo Grau':
            instancia_xpath = self.find(self.grau2_xpath)

        elif instancia == 'Turmas Recursais':
            instancia_xpath = self.find(self.grau3_xpath)

        else:
            raise Exception(f'A instância selecionado é {instancia}')

        return instancia_xpath

    def check_avaibled(
        self,
        instancia: Literal['Primeiro Grau', 'Segundo Grau', 'Turmas Recursais'],
    ):
        """
        Avalia se a opção está disponível para o usuário logado.
        Pode haver instâncias não disponíveis
        """
        instancia_xpath = self._get_xpath_instance(instancia=instancia)
        # instancia = self.get_selected_option()
        # Valida permissões
        # if self.attribute(instancia_xpath, 'disabled') == 'true':
        if instancia_xpath.get_attribute('disabled') == 'true':
            raise RuntimeError(
                f'Usuário não tem permissão de acesso à instância.'
            )

    def set_option(
        self,
        instancia: Literal['Primeiro Grau', 'Segundo Grau', 'Turmas Recursais'],
    ) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        # Avalia se opção está disponível
        self.check_avaibled(instancia=instancia)

        # Avalia Instância Atualmente Selecionada
        instancia_atual = self.get_selected_option()
        # print(f'Qual a instância atual: {instancia_atual}')
        if instancia_atual not in [
            'Primeiro Grau',
            'Segundo Grau',
            'Turmas Recursais',
        ]:
            raise ValueError(
                f'A instância precisa ser "Primeiro Grau", "Segundo Grau" ou "Turmas Recursais"\n'
                f'Está definido {instancia_atual}'
            )

        # Avalia Instância Desejada
        if instancia not in [
            'Primeiro Grau',
            'Segundo Grau',
            'Turmas Recursais',
        ]:
            raise ValueError(
                f'A instância precisa ser "Primeiro Grau", "Segundo Grau" ou "Turmas Recursais"\n'
                f'Está definido {instancia}'
            )

        # Clica
        instancia_xpath = self._get_xpath_instance(instancia=instancia)
        # print(instancia_xpath.get_attribute(name='id'))
        instancia_xpath.click()

        # Notei que ao mudar para 2º Grau, às vezes o formulário não atualizava
        # Dai se aperta o botão "Consultar", atualiza...
        # Portanto, fiz essa rotina abaixo
        text_label = self.get_text(locator=self.label_foro_secao)
        # print(f'O texto inicial é "{text_label}"')
        tentativa = 0
        n_tentativas = 5
        while (
            instancia in ('Segundo Grau', 'Turmas Recursais')
            and text_label != 'Seção'
            and tentativa <= n_tentativas
        ):
            self.click(locator=self.btn_consultar)
            # print('Cliquei em Consultar')
            time.sleep(5)
            text_label = self.get_text(locator=self.label_foro_secao)
            tentativa += 1
            # print(f'O texto do while é "{text_label}"')


class Foro(InputModelSearchSet):
    """
    Representa o campo de *input* "Foro", que define
    qual o Foro que disponibilizou a intimação.

    Utilizado apenas em intimações de [instância] 1º grau.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelForo"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="nmForo"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.nmForo"]')
        self.lista_opcoes = None

        # Só tem em Primeiro Grau
        intim_inst = Instancia(driver=driver)
        if intim_inst.get_selected_option() != 'Primeiro Grau':
            raise Exception(
                f'Só se define "{self.__class__.__name__}" quando em instância de "Primeiro Grau"'
            )


class Vara(InputModelSearchSet):
    """
    Representa o campo de *input* "Vara", que define
    qual a Vara que disponibilizou a intimação.

    Utilizado apenas em intimações de [instâncias de] 1º grau.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelVara"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="nmVara"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.nmVara"]')
        self.lista_opcoes = None

        # Só tem em Primeiro Grau
        intim_inst = Instancia(driver=driver)
        if intim_inst.get_selected_option() != 'Primeiro Grau':
            raise Exception(
                f'Só se define "{self.__class__.__name__}" quando em instância de "Primeiro Grau"'
            )


class Especializacao(InputModelSearchSet):
    """
    Representa o campo de *input* "Especialização", que define
    qual a Especialização das intimações disponibilizadas.
    """

    def __init__(self, driver) -> None:
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelEspecialidade"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="nmEspecialidade"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.nmEspecialidade"]')
        self.lista_opcoes = None
        self.checkbox = (
            By.XPATH,
            '//td[@id="labelEspecialidade"]/following-sibling::td//tr//input[@name="dadosConsulta.filtrarApenasAtoSemEspecialidadeDefinida"]',
        )
        self.check_checkbox = (
            By.XPATH,
            '//td[@id="labelEspecialidade"]/following-sibling::td//tr//input[@id="entity.ato.especialidade.cdEspecialidade"]',
        )

    def apenas_processos_sem_especializacao(self) -> None:
        """
        Habilita o *checkbox* "Filtrar Processos sem Especialização definida".
        """
        attrib = self.attribute(locator=self.check_checkbox, attribute='class')
        # print('Especializa', attrib)
        if attrib.strip() != 'disabled':
            self.click(self.checkbox)


class Cargo(InputModelSearchSet):
    """
    Representa o campo de *input* "Cargo", que define
    qual o Cargo responsável pelas intimações disponibilizadas.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelCargo"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.iframe = (By.XPATH, '//iframe[@id="layerFormConsulta"]')
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="deCargo"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.deCargo"]')
        self.lista_opcoes = None
        self.checkbox = (
            By.XPATH,
            '//td[@id="labelCargo"]/following-sibling::td//tr//input[@name="dadosConsulta.filtrarApenasAtoSemCargo"]',
        )
        self.check_checkbox = (
            By.XPATH,
            '//td[@id="labelCargo"]/following-sibling::td//tr//input[@id="entity.ato.cargo.cdCargo"]',
        )

    def apenas_processos_sem_cargo(self) -> None:
        """
        Habilita o *checkbox* "Filtrar Processos sem Cargo definido".
        """
        attrib = self.attribute(locator=self.check_checkbox, attribute='class')
        # print('Cargo', attrib)
        if attrib.strip() != 'disabled':
            self.click(self.checkbox)


class Processo(PageElement):
    """
    Representa os campos de *input* "Processo", que define
    qual o processo que pertence à intimação disponibilizada.
    """

    processo_xpath = (By.XPATH, '//input[@id="identity.nuProcessoFormat"]')

    def write(self, texto: str) -> None:
        """
        Define o Número do Processo
        """
        self.set(locator=self.processo_xpath, value=texto)

    def clear(self) -> None:
        """
        Limpa o campo text
        """
        self.set(locator=self.processo_xpath, value='')


class NaturezaComunicacao(PageElement):
    """
    Representa o *radio button* "Natureza da comunicação".
    """

    # btn_consultar = (By.XPATH, '//input[@id="pbConsultar"]')
    # label_foro_secao = (By.XPATH, '//td[@id="labelForo"]/label')

    def set_option(
        self, natureza: Literal['Intimação', 'Citação', 'Ambas']
    ) -> None:
        """
        Define a opção para preenchimento do campo.
        """

        dd_natureza = {
            'Intimação': 'radioIntimacao',
            'Citação': 'radioCitacao',
            'Ambas': 'radioIntimacaoCitacao',
        }

        natureza_xpath = self.find(
            (By.XPATH, f'//input[@id="{dd_natureza[natureza]}"]')
        )
        natureza_xpath.click()


class Secao(InputModelSearchSet):
    """
    Representa o campo de *input* "Seção", que define
    qual a Seção que disponibilizou a intimação.

    Utilizado apenas em intimações de [instância] 2º grau.
    Equivalente ao "Foro" utilizado no 1º grau.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelForo"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="deCartorio"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.deCartorio"]')
        self.lista_opcoes = None

        # Só tem em Segundo Grau ou Turmas Recursais
        intim_inst = Instancia(driver=driver)
        instancia = intim_inst.get_selected_option()
        if instancia not in ['Segundo Grau', 'Turmas Recursais']:
            raise Exception(
                f'Só se define "{self.__class__.__name__}" quando em instância de "Segundo Grau" ou "Turmas Recursais"\n'
                f'Está definido "{instancia}"'
            )


class OrgaoJulgador(InputModelSearchSet):
    """
    Representa o campo de *input* "Órgao Julgador", que define
    qual o Órgao Julgador que disponibilizou a intimação.

    Utilizado apenas em intimações de [instância] 2º grau.
    Equivalente à "Vara" utilizado no 1º grau.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td[@id="labelVara"]/following-sibling::td//tr//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="nmVara"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.nmVara"]')
        self.lista_opcoes = None

        # Só tem em Segundo Grau ou Turmas Recursais
        intim_inst = Instancia(driver=driver)
        instancia = intim_inst.get_selected_option()
        if instancia not in ['Segundo Grau', 'Turmas Recursais']:
            raise Exception(
                f'Só se define "{self.__class__.__name__}" quando em instância de "Segundo Grau" ou "Turmas Recursais"\n'
                f'Está definido "{instancia}"'
            )


class Classe(InputModelTree):
    """
    Representa o campo de *input* "Classe", que define
    qual a Classe que pertence a intimação disponibilizada.
    """

    def __init__(self, driver):
        super().__init__(driver)
        # O xPath do "Abre Consulta" é um pouco diferente do padrão
        self.abre_consulta = (
            By.XPATH,
            '//td[@id=""]/following-sibling::td//tr//img[@id="botaoProcurar_classes" and @title="Abre a consulta"]',
        )
        self.btn_selecionar = (
            By.XPATH,
            '//div[@id="classes_treeSelectContainer"]//input[@class="spwBotaoDefaultGrid" and @value="Selecionar"]',
        )
        self.btn_procurar = (
            By.XPATH,
            '//div[@id="classes_treeSelectContainer"]//input[@id="filtroButton"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//ul[@id="classes_tree"]//span[@class="node selectable checkable Unchecked"]',
        )
        self.input_opcao = (
            By.XPATH,
            '//input[@id="classes_treeSelectFilter"]',
        )  #
        self.lista_opcoes = None


class AssuntoPrincipal(InputModelTree):
    """
    Representa o campo de *input* "Assunto Principal", que define
    qual o Assunto Principal que pertence à intimação disponibilizada.
    """

    def __init__(self, driver):
        super().__init__(driver)
        # O XPath do "Abre Consulta" é um pouco diferente do padrão
        self.abre_consulta = (
            By.XPATH,
            '//td[@id=""]/following-sibling::td//tr//img[@id="botaoProcurar_assuntos" and @title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//ul[@id="assuntos_tree"]//span[@class="node selectable checkable Unchecked"]',
        )
        self.input_opcao = (
            By.XPATH,
            '//input[@id="assuntos_treeSelectFilter"]',
        )
        self.btn_selecionar = (
            By.XPATH,
            '//div[@id="assuntos_treeSelectContainer"]//input[@class="spwBotaoDefaultGrid" and @value="Selecionar"]',
        )
        self.lista_opcoes = None


class Area(PageElement):
    """
    Representa o menu *dropdown* "Área", que define
    qual a Área que pertence à intimação disponibilizada.
    """

    selector = (By.XPATH, '//select[@id="flArea"]')

    def get_options(self, options_delete=None) -> List[str]:
        """
        Obtem opções de preenchimento.
        """

        return self.create_select_options(self.selector, options_delete)

    def set_option(self, option) -> None:
        """
        Define a opção para preenchimento do campo.
        """
        list_options = self.get_options()
        if option not in list_options:
            raise ValueError(f'Precisa ser uma opção {list_options}')

        self.select(self.selector, option)


class ConsultarIntimacoes(PageElement):
    """
    Representa o botão "Consultar"
    """

    btn_consultar = (
        By.XPATH,
        '//input[@id="pbConsultar"]',
    )
    table_xpath = (
        By.XPATH,
        '//table[@id="Row" and @class="spwTabelaGrid"]',
    )

    def consultar(self) -> None:
        """
        Aperta o botão "Consultar"
        """
        self.click(locator=self.btn_consultar)

    def check_table(self):
        """
        Avalia se tem resultados
        """
        attrib = self.attribute(locator=self.table_xpath, attribute='class')
        if attrib == 'spwTabelaGrid':
            return True
        else:
            return False


# class CheckError(PageElement):
#     """
#     Avalia se o e-SAJ está com algum erro apresentado.
#     """

#     msg_erro = (
#         By.XPATH,
#         '//*[contains(text(),"Não foi possível executar esta operação. Tente novamente mais tarde.")]',
#     )
#     msg_resultado = (
#         By.XPATH,
#         '//div[@id="spwTabelaMensagem"]//table[@class="tabelaMensagem"]',
#     )

#     def has_errors(self) -> bool:
#         """
#         Avalia erro "Não foi possível executar esta operação. Tente novamente mais tarde".
#         """
#         try:
#             # Find Errors
#             tags = self.find_all(locator=self.msg_erro, wait=3)

#             #
#             if len(tags) > 0:
#                 # Mensagem
#                 warnings.warn(
#                     message=f'\n'
#                     f'e-SAJ com problemas!!.\n'
#                     f'"Não foi possível executar esta operação. Tente novamente mais tarde"'
#                     f'\n',
#                     category=RuntimeWarning,
#                 )
#                 return True

#             else:
#                 print(f'Erro: Encontrou Tag!? {len(tags)} registros')
#                 return False

#         except:
#             return False

#     def tem_resultado(self) -> bool:
#         """
#         Avalia o resultado da pesquisa

#         - `True` significa que tem regstros
#         - `False` significa que NÃO tem regstros
#         """
#         try:
#             # Find Errors
#             tags = self.find_all(locator=self.msg_resultado, wait=3)
#             print(tags[0].text)

#             # Se tem
#             if len(tags) == 1 and (
#                 tags[0].text
#                 == 'Atenção\n  Não foram encontrados registros de atos para os parâmetros informados.'
#             ):
#                 # Mensagem
#                 warnings.warn(
#                     message=f'\n'
#                     f'A pesquisa não retornou nenhum resultado.\n'
#                     f'\n',
#                     category=RuntimeWarning,
#                 )
#                 # Se encontra a mensagem, significa que não tem resultado
#                 return False

#             else:
#                 #print('dsdsdsd')
#                 print(print(tags[0].text))
#                 return False

#         except:
#             # Se não encontra a mensagem, significa que tem resultado!
#             return True


class CheckError(PageElement):
    """
    Avalia se o e-SAJ está com algum erro apresentado.
    """

    msg_erro = (
        By.XPATH,
        '//*[contains(text(),"Não foi possível executar esta operação. Tente novamente mais tarde.")]',
    )

    @disable_logging
    def has_errors(self) -> bool:
        """
        Avalia erro "Não foi possível executar esta operação. Tente novamente mais tarde".
        """
        try:
            # Find Errors
            tags = self.find_all(locator=self.msg_erro, wait=3)

            #
            if len(tags) > 0:
                # Mensagem
                logging.warning(
                    msg=f'e-SAJ com problemas: "Não foi possível executar esta operação. Tente novamente mais tarde"'
                )
                return True

            else:
                print(f'Erro: Encontrou Tag!? {len(tags)} registros')
                return False

        except:
            return False


class CheckResults(PageElement):
    """
    Avalia os resultados do e-SAJ.
    """

    msg_resultado = (
        By.XPATH,
        '//div[@id="spwTabelaMensagem"]//table[@class="tabelaMensagem"]',
    )

    table_xpath = (
        By.XPATH,
        '//table[@id="Row" and @class="spwTabelaGrid"]',
    )

    @disable_logging_property
    def tipo_resultado(self):
        """
        Define qual o tipo de resultado
        """

        tipo_resultado_var = None
        n_tentativa = 0

        while tipo_resultado_var is None and n_tentativa <= 10:
            # Tenta encontrar mensagem
            try:
                msg = self.find(locator=self.msg_resultado, wait=0.01)

            except:
                msg = None

            # Tenta encontrar tabela
            try:
                tab = self.find(locator=self.table_xpath, wait=0.01)

            except:
                tab = None

            if tab == None and msg == None:
                tipo_resultado_var = None

            elif tab == None and msg != None:
                tipo_resultado_var = 'Mensagem'

            elif tab != None and msg == None:
                tipo_resultado_var = 'Tabela'

            else:
                tipo_resultado_var = None

            n_tentativa += 1
        return tipo_resultado_var

    @disable_logging_property
    def messages(self) -> list | None:
        """
        Obtem a lista de mensagens
        """
        # Obtem o tipo de Resultado
        tipo_resultado_var = self.tipo_resultado

        # Se é Mensagem
        if tipo_resultado_var == 'Mensagem':
            try:
                tags = self.find_all(locator=self.msg_resultado, wait=3)
                list_messages = []
                for tag in tags:
                    list_values = tag.text.strip().split('\n')
                    list_values = [x.strip() for x in list_values]
                    list_messages.append(': '.join(list_values))

                # Excluí mensagem em braco (usada em formação do HTML)
                list_messages = [msg for msg in list_messages if msg]
                # print('Lista', list_messages)
                return list_messages

            # Se não encontra, é lista vazia
            except Exception as e:
                # print(f"Erro: {e}")
                list_messages = []
                return list_messages

        # Se é Tabela
        elif tipo_resultado_var == 'Tabela':
            list_messages = []
            return list_messages

    @disable_logging
    def tem_resultado(self) -> bool:
        """
        Avalia o resultado da pesquisa

        - `True` significa que tem registros
        - `False` significa que NÃO tem regstros
        """
        messages = self.messages
        # print('A lista de msg é: ', messages)

        # Se não encontra qualquer mensagem, significa que tem resultado!
        if messages == []:
            return True

        # Se encontra alguma mensagem, não tem resultado
        # Ou se faz necessário entender o que ocorre
        elif messages != []:

            if (
                'Atenção: Não foram encontrados registros de atos para os parâmetros informados.'
                in messages
            ):

                # Mensagem
                # logging.warning(msg='A pesquisa não retornou nenhum resultado.')

                # Se encontra a mensagem, significa que não tem resultado
                return False

            else:
                print('Definir erros:', messages)
                raise Exception


class Tabela(PageElement):
    """
    Representa a tabela de resultados
    """

    table_xpath = (By.XPATH, '//table[@id="Row" and @class="spwTabelaGrid"]')

    def get_xpath_rows(self):
        # Cria Lista de xPaths
        list_xpath = [
            f'//table[@id="Row" and @class="spwTabelaGrid"]//tr[@id="linhaRow_{x}"]'
            for x in range(0, 100)
        ]

        # Linhas
        list_xpath_or = ' | '.join(list_xpath)
        rows_xpath = self.find_all(locator=(By.XPATH, list_xpath_or))

        # Results
        # print(f'São {len(rows_xpath)} linhas')
        return rows_xpath

    def get_table(
        self,
        *args,
        **kwargs,
    ) -> (
        List[RecebimentoIntimacoes1Grau]
        | List[RecebimentoIntimacoes2Grau]
        | List[ConsultaIntimacoes2Grau]
    ):
        """
        Obtém a tabela com 20 registros, sendo essa a paginação padrão do e-SAJ.
        """
        # Vê qual a instancia
        # intim_tipo = Instancia(driver=self)
        # instancia = intim_tipo.get_selected_option()
        intim_inst = Instancia(driver=self.driver)
        instancia = intim_inst.get_selected_option()

        # Cria lista com os xPaths das linhas
        rows_xpath = self.get_xpath_rows()

        list_dd = []
        for row in rows_xpath:
            # Pega o Número do Processo
            x_processo = (
                By.XPATH,
                './/td//div[@name_="ato.nuProcessoFormat"]',
                row,
            )

            # Atributos
            x_disponibilizacao = (
                By.XPATH,
                './/td//div[@name_="ato.dtInclusao"]',
                row,
            )
            x_dt_intimacao = (By.XPATH, './/td//div[@name_="dtIntimacao"]', row)
            x_prazo = (By.XPATH, './/td//div[@name_="nuDiasPrazo"]', row)
            x_classe = (
                By.XPATH,
                './/td//div[@name_="classeEAssuntoPrincipaisFormatado"]',
                row,
            )
            x_recebido_por = (
                By.XPATH,
                './/td//div[@name_="usuarioIntimacao.nmUsuario"]',
                row,
            )
            x_mov = (By.XPATH, './/td//div[@name_="ato.deTipoMvProcesso"]', row)
            x_foro = (By.XPATH, './/td//div[@name_="ato.foro.nmForo"]', row)
            x_vara = (By.XPATH, './/td//div[@name_="ato.vara.nmVara"]', row)
            x_sec = (
                By.XPATH,
                './/td//div[@name_="ato.cartorio.deCartorio"]',
                row,
            )
            x_org = (By.XPATH, './/td//div[@name_="ato.vara.nmVara"]', row)
            x_esp = (
                By.XPATH,
                './/td//div[@name_="ato.especialidade.nmEspecialidade"]',
                row,
            )
            x_car = (By.XPATH, './/td//div[@name_="ato.cargo.deCargo"]', row)

            path = urlparse(url=self.driver.current_url).path

            # TODO Mapear tarjas / Cores
            # Notei que existem processos vermelhos no "Consulta"
            # Recebimento de Intimações Eletrônicas ()
            if path == '/intimacoesweb/abrirConsultaAtosNaoRecebidos.do':
                # print(f'Endpoint{path}')
                if instancia == 'Primeiro Grau':
                    list_dd.append(
                        RecebimentoIntimacoes1Grau(
                            disponibilizacao=self.get_text(x_disponibilizacao),
                            prazo_processual=self.get_text(x_prazo),
                            numero_processo=self.get_text(x_processo),
                            classe_assunto_principal=self.get_text(x_classe),
                            movimentacao=self.get_text(x_mov),
                            foro=self.get_text(x_foro),
                            vara=self.get_text(x_vara),
                            especializacao=self.get_text(x_esp),
                            cargo=self.get_text(x_car),
                        )
                    )

                # Quando for Recebimento
                if instancia in ['Segundo Grau', 'Turmas Recursais']:
                    list_dd.append(
                        RecebimentoIntimacoes2Grau(
                            disponibilizacao=self.get_text(x_disponibilizacao),
                            prazo_processual=self.get_text(x_prazo),
                            numero_processo=self.get_text(x_processo),
                            classe_assunto_principal=self.get_text(x_classe),
                            movimentacao=self.get_text(x_mov),
                            secao=self.get_text(x_sec),
                            orgao_julgador=self.get_text(x_org),
                            especializacao=self.get_text(x_esp),
                            cargo=self.get_text(x_car),
                        )
                    )

            # Recebimento de Intimações Eletrônicas
            elif path == '/intimacoesweb/consultarAtosNaoRecebidos.do':
                if instancia == 'Primeiro Grau':
                    list_dd.append(
                        RecebimentoIntimacoes1Grau(
                            disponibilizacao=self.get_text(x_disponibilizacao),
                            prazo_processual=self.get_text(x_prazo),
                            numero_processo=self.get_text(x_processo),
                            classe_assunto_principal=self.get_text(x_classe),
                            movimentacao=self.get_text(x_mov),
                            foro=self.get_text(x_foro),
                            vara=self.get_text(x_vara),
                            especializacao=self.get_text(x_esp),
                            cargo=self.get_text(x_car),
                        )
                    )

                # Quando for Recebimento
                if instancia in ['Segundo Grau', 'Turmas Recursais']:
                    list_dd.append(
                        RecebimentoIntimacoes2Grau(
                            disponibilizacao=self.get_text(x_disponibilizacao),
                            prazo_processual=self.get_text(x_prazo),
                            numero_processo=self.get_text(x_processo),
                            classe_assunto_principal=self.get_text(x_classe),
                            movimentacao=self.get_text(x_mov),
                            secao=self.get_text(x_sec),
                            orgao_julgador=self.get_text(x_org),
                            especializacao=self.get_text(x_esp),
                            cargo=self.get_text(x_car),
                        )
                    )

            # Consulta de Intimações Recebidas
            elif path == '/intimacoesweb/consultarAtosRecebidos.do':
                if instancia in ['Segundo Grau', 'Turmas Recursais']:
                    list_dd.append(
                        ConsultaIntimacoes2Grau(
                            disponibilizacao=self.get_text(x_disponibilizacao),
                            data_intimacao=self.get_text(x_dt_intimacao),
                            prazo_processual=self.get_text(x_prazo),
                            numero_processo=self.get_text(x_processo),
                            classe_assunto_principal=self.get_text(x_classe),
                            recebido_por=self.get_text(x_recebido_por),
                            movimentacao=self.get_text(x_mov),
                            # secao=self.get_text(x_sec),
                            # orgao_julgador=self.get_text(x_org),
                            especializacao=self.get_text(x_esp),
                            cargo=self.get_text(x_car),
                        )
                    )
            else:
                print(path)

        return list_dd

    def select_processos(
        self,
        seleciona_processos: List,
        *args,
        **kwargs,
    ):
        """
        Seleciona os processos na tabela
        """
        # Cria lista com os xPaths das linhas
        rows_xpath = self.get_xpath_rows()

        for row in rows_xpath:
            # Pega o Número do Processo
            x_processo = (
                By.XPATH,
                './/td//div[@name_="ato.nuProcessoFormat"]',
                row,
            )
            processo = self.get_text(x_processo)

            # Se o Processo está na lista, seleciona
            if processo in seleciona_processos:
                x_check_checkbox = (
                    By.XPATH,
                    './/td//input[@type="checkbox"]/../following-sibling::td[@style="display: none"]/input[@name_]',
                    row,
                )
                # Avalia se o CheckBox está marcado. Em caso negativo, clica!
                # Se não encontra o atributo "name", o processo não está selecionado
                if 'cdAto' not in self.attribute(
                    locator=x_check_checkbox, attribute='name'
                ):
                    # Clica
                    x_checkbox = (
                        By.XPATH,
                        './/td//input[@type="checkbox"]',
                        row,
                    )
                    self.click(locator=x_checkbox)

    def count_selected_processos(
        self,
        type_result: Literal['lista', 'numero de registros'],
        *args,
        **kwargs,
    ):
        """
        Conta o número de registros selecionados na tabela
        """
        # Cria lista com os xPaths das linhas
        rows_xpath = self.get_xpath_rows()

        list_dd = []
        for row in rows_xpath:
            # Pega o Número do Processo
            x_processo = (
                By.XPATH,
                './/td//div[@name_="ato.nuProcessoFormat"]',
                row,
            )
            processo = self.get_text(x_processo)

            # Se o Processo está na lista, seleciona
            x_check_checkbox = (
                By.XPATH,
                './/td//input[@type="checkbox"]/../following-sibling::td[@style="display: none"]/input[@name_]',
                row,
            )
            # Avalia se o Checkbox está marcado.
            # Se encontra o atributo "name", o processo está selecionado
            if 'cdAto' in self.attribute(
                locator=x_check_checkbox, attribute='name'
            ):
                list_dd.append(processo)

        if type_result == 'lista':
            return list_dd

        elif type_result == 'numero de registros':
            return len(list_dd)

    def count_all_processos(
        self,
        type_result: Literal['lista', 'numero de registros'],
        *args,
        **kwargs,
    ):
        """
        Conta o número de registros na tabela, selecionados ou não
        """

        rows_xpath = self.get_xpath_rows()

        if type_result == 'lista':
            return rows_xpath

        elif type_result == 'numero de registros':
            return len(rows_xpath)


class Acoes(PageElement):
    """
    Represante os botões que promovem "Ações", tais como:
    - Receber
    - Especializar
    - Exportar
    """

    # btn_define = (
    #     By.XPATH,
    #     '//table[@class="secaoBotoesBody"]//input[@id="definirEspecialidade"]',
    # )
    # btn_define = (
    #     By.XPATH,
    #     '//table[@class="secaoBotoesBody"]//input[@id="definirEspecialidade"]',
    # )
    btn_define = (
        By.XPATH,
        '//table[@class="secaoBotoesBody"]//input[@id="definirEspecialidade" and @value="Definir Especialização e Cargo"]',
    )

    btn_export = (
        By.XPATH,
        '//table[@class="secaoBotoesBody"]//input[@id="exportarArquivo"]',
    )
    btn_receber = (
        By.XPATH,
        '//table[@class="secaoBotoesBody"]//input[@id="pbReceberAtos"]',
    )

    def definir_especializacao_cargo(self) -> None:
        """
        Aperta botão para "Definir Especialização e Cargo"
        """
        # Avaliar se algum processo está selecionado
        self.click(self.btn_define)

    def export_csv(self, *args, **kwargs) -> Path:
        """
        Aperta botão para "Exportar para Arquivo".
        TODO: Fazer funcionar no Chrome
        """

        # Download Path
        download_path = kwargs.get('download_path', False)
        if download_path is False:
            # Temp Path
            temp_path = tempfile.gettempdir()
            project_temp_path = Path(temp_path) / config.TEMP_PATH_NAME

            # Scrapy Path
            scrapy_path = project_temp_path / 'scrapy'
            scrapy_path.mkdir(exist_ok=True, parents=True)

            # Cria Pasta
            download_path = scrapy_path / 'download'
            download_path.mkdir(exist_ok=True, parents=True)
            print('A pasta de download de dados é: ', download_path)

        # Deleta tudo
        list_csvs = list(download_path.rglob('*.csv'))
        start_time = time.time()
        while len(list_csvs) != 0:
            [x.unlink() for x in list_csvs]
            time.sleep(1)
            list_csvs = list(download_path.rglob('*.csv'))

            # Verifica se o tempo limite de 60 segundos foi atingido
            if time.time() - start_time > 60:
                raise Exception(
                    "Tempo limite atingido. O delete não foi concluído."
                )

        # Faz Download
        self.click(self.btn_export)
        list_csvs = list(download_path.rglob('*.csv'))
        start_time = time.time()
        while len(list_csvs) != 1:
            time.sleep(1)
            list_csvs = list(download_path.rglob('*.csv'))

            # Verifica se o tempo limite de 60 segundos foi atingido
            if time.time() - start_time > 60:
                raise Exception(
                    "Tempo limite atingido. O download não foi concluído."
                )

        #
        if len(list(download_path.rglob('*.csv'))) == 1:
            return list(download_path.rglob('*.csv'))[0]

        else:
            raise Exception(
                f'Era pra ter algum um arquivo na pasta\n{download_path}'
            )

    def receber(self) -> None:
        """
        Aperta botão para "Receber Selecionados"
        """
        self.click(self.btn_receber)


if __name__ == '__main__':
    pass



"""
Módulo para usar driver do FireFox

Michel Metran
Data: mar.2023
Atualizado em: mar.2023
"""

import tempfile
from pathlib import Path

import requests
from selenium import webdriver
from selenium.webdriver.firefox.options import Options as FirefoxOptions
from selenium.webdriver.firefox.service import Service as FirefoxService

from pyesaj.scraper.webdriver import config
from pyesaj.scraper.webdriver import gecko


class Firefox(webdriver.Firefox):
    """
    Cria driver customizado do Selenium

    :param webdriver: _description_
    :type webdriver: _type_
    """

    def __init__(
            self,
            # driver_path: Path,
            # logs_path: Path,
            # down_path: Path,
            *args,
            **kwargs,
    ):
        """
        - verify_ssl
        - headless
        - download_path

        :param my_driver_path: _description_
        :type my_driver_path: pathlib
        :param my_logs_path: _description_
        :type my_logs_path: pathlib
        """

        # Parameters
        verify_ssl = kwargs.get('verify_ssl', True)
        headless = kwargs.get('headless', False)
        self.download_path = kwargs.get('download_path', False)

        # Temp Path
        temp_path = tempfile.gettempdir()
        project_temp_path = Path(temp_path) / config.TEMP_PATH_NAME

        # Scrapy Path
        scrapy_path = project_temp_path / 'scrapy'
        scrapy_path.mkdir(exist_ok=True, parents=True)

        # Driver Path
        self.driver_path = scrapy_path / 'driver'
        self.driver_path.mkdir(exist_ok=True, parents=True)

        # Logs Path
        self.logs_path = scrapy_path / 'logs'
        self.logs_path.mkdir(exist_ok=True, parents=True)

        # Services
        geckodriver = gecko.Gecko()
        gecko_path = geckodriver.get_path_geckodriver(verify_ssl=verify_ssl)

        # Logs
        logs_filepath = self.logs_path / 'geckodriver.log'

        # Services
        my_service = FirefoxService(
            executable_path=gecko_path, log_path=logs_filepath
        )

        # Options
        my_options = FirefoxOptions()
        if headless:
            my_options.add_argument('--headless')

        # Download Path
        if self.download_path is False:
            # Cria Pasta
            self.download_path = scrapy_path / 'download'
            self.download_path.mkdir(exist_ok=True, parents=True)

        # Define pasta de Download
        my_options.set_preference(
            'browser.download.dir', str(self.download_path)
        )

        my_options.set_preference('intl.accept_languages', 'pt-BR, pt')
        my_options.set_preference('browser.download.folderList', 2)
        my_options.set_preference(
            'browser.download.manager.showWhenStarting', False
        )
        my_options.set_preference('pdfjs.disabled', True)
        my_options.set_preference('plugin.scan.Acrobat', '99.0')
        my_options.set_preference('plugin.scan.plid.all', False)
        my_options.set_preference(
            'browser.helperApps.showOpenOptionForPdfJS', False
        )
        my_options.set_preference('browser.download.forbid_open_with', True)
        my_options.set_preference(
            'browser.helperApps.neverAsk.saveToDisk',
            'application/octet-stream, application/pdf',
        )
        # Quando é necessário fazer repost about: config
        # https://superuser.com/questions/1410598/disable-firefox-must-send-information-that-will-repeat-any-action-dialog-box
        my_options.set_preference(
            'dom.confirm_repost.testing.always_accept', True
        )

        # Driver
        # my_driver = super(Driver, self)
        # my_driver.__init__(service=my_service, options=my_options)
        super().__init__(service=my_service, options=my_options)
        self.maximize_window()

    def add_extension_xpath(self) -> None:
        """
        Adiciona xPath extension
        """

        # Temp Path
        temp_path = tempfile.gettempdir()
        project_temp_path = Path(temp_path) / config.TEMP_PATH_NAME

        # Scrapy Path
        scrapy_path = project_temp_path / 'scrapy'
        scrapy_path.mkdir(exist_ok=True, parents=True)

        # Adds Path
        adds_path = scrapy_path / 'adds'
        adds_path.mkdir(exist_ok=True, parents=True)

        # Add-ons Xpath
        xpath_path = adds_path / 'xpath.xpi'
        xpath_path = xpath_path.absolute().resolve()

        # Download
        if not xpath_path.is_file():
            r = requests.get(
                url=config.URL_ADDONS_XPATH, timeout=60, verify=False
            )

            with open(xpath_path, 'wb') as f:
                f.write(r.content)

        # Add
        self.install_addon(str(xpath_path), temporary=True)


if __name__ == '__main__':
    import time

    # from pyesaj import paths

    # Instancia Driver
    driver = Firefox(headless=False, verify_ssl=False, )

    # Add xPath Extension
    driver.add_extension_xpath()

    driver.get(url='https://www.uol.com.br/')
    time.sleep(3)
    driver.quit()

    gecko = gecko.Gecko()
    gecko_p = gecko.get_path_geckodriver()
    print(gecko_p)



"""
Módulo para criar decorator para desabilitar o login em funções.

Isso se faz necessário pois existem funções que usam a tentativa e erro de encontrar um elemento, para trazer um resultado.
Por exemplo, "não tem resultado" é um retorno quando a função não encontra elementos.

Logo, é necessário desabilitar o logging "não encontrou elementos" para essas funções, deixando-a apenas em funções que é necessário encontrar um elemento, senão é erro....

https://stackoverflow.com/questions/7341064/disable-logging-per-method-function

Michel Metran
Data: 18.12.2024
Atualizado em: 18.12.2024
"""

import functools
import logging
from functools import wraps


def disable_logging(func):
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        logging.disable(logging.CRITICAL)
        result = func(*args, **kwargs)
        logging.disable(logging.NOTSET)
        return result

    return wrapper


def disable_logging_property(func):
    @property
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        logging.disable(logging.CRITICAL)
        result = func(*args, **kwargs)
        logging.disable(logging.NOTSET)
        return result

    return wrapper


if __name__ == '__main__':
    logging.basicConfig(
        level=logging.DEBUG,
        format=('%(levelname)s: %(funcName)s(): %(message)s'),
    )
    log = logging.getLogger()

    some_value = 1

    # log.debug("Here's an interesting value: %r" % some_value)
    # log.info("Going great here!")

    # @disable_logging
    def my_func1():
        log.debug("This is my_func1()")

    my_func1()



"""
Módulo para pesquisar processos
"""

import time

from selenium import webdriver

import pyesaj as esaj
from pyesaj.scraper.params.processo.input import PesquisaProcessos


def pesquisa(driver: webdriver.Firefox, dados: PesquisaProcessos) -> None:
    """
    Pesquisa pelas intimações
    """
    # Vai para página
    esaj.scraper.page.processo.ConsultaProcesso(driver=driver, grau=dados.instancia)

    # Define Parâmetro de Pesquisa
    por = esaj.scraper.page.processo.ConsultaPor(driver=driver)
    # print(por.get_options())
    por.set_option(option=dados.consultar_por)

    # Número
    if dados.numero_unificado is not None:
        num = esaj.scraper.page.processo.NumeroUnificado(driver=driver)
        num.define_numero(numero=dados.numero_unificado)

    if dados.nome_parte is not None:
        nome = esaj.scraper.page.processo.NomeParte(driver=driver)
        nome.set_nome_parte(nome=dados.nome_parte)
        if dados.nome_parte_pesquisa_nome_completo is True:
            nome.pesquisa_nome_completo()

    # Documento da Parte
    if dados.documento_parte is not None:
        doc = esaj.scraper.page.processo.DocumentoParte(driver=driver)
        doc.set_documento(documento=dados.documento_parte)

    # Nome do Advogado
    if dados.nome_advogado is not None:
        nome = esaj.scraper.page.processo.NomeAdvogado(driver=driver)
        nome.set_nome_adv(nome=dados.nome_advogado)
        if dados.nome_advogado_pesquisa_nome_completo is True:
            nome.pesquisa_nome_completo()

    # OAB
    if dados.oab is not None:
        oab = esaj.scraper.page.processo.OAB(driver=driver)
        oab.set_documento(documento=dados.oab)
        # if dados.nome_advogado_pesquisa_nome_completo is True:
        #     nome.pesquisa_nome_completo()

    # Número da Carta Precatória na Origem
    if dados.n_carta_precatoria_origem is not None:
        carta = esaj.scraper.page.processo.NumCartaPrecatoria(driver=driver)
        carta.set_documento(documento=dados.n_carta_precatoria_origem)

    # Nº do Documento na Delegacia
    if dados.n_documento_delegacia is not None:
        deleg = esaj.scraper.page.processo.NumDocDelegacia(driver=driver)
        deleg.set_documento(documento=dados.n_documento_delegacia)

    # CDA
    if dados.cda is not None:
        cda = esaj.scraper.page.processo.CDA(driver=driver)
        cda.set_documento(documento=dados.cda)

    # Consultar
    con = esaj.scraper.page.processo.Consultar(driver=driver)
    con.consultar(grau=dados.instancia)


if __name__ == '__main__':
    # import os

    # from dotenv import load_dotenv

    # Credenciais
    # load_dotenv()
    # USERNAME = os.getenv('USERNAME_TJSP')
    # PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver2 = esaj.scraper.webdriver.Firefox(verify_ssl=False)

    # Faz Login
    # log = esaj.page.Login(driver=driver)
    # log.login(username=USERNAME, password=PASSWORD)

    # Define
    proced = PesquisaProcessos(
        instancia='Segundo Grau',
        # consultar_por='Número do Processo',
        consultar_por='Nome da parte',
        # consultar_por='CDA',
        # numero_unificado='0123479-07.2012.8.26.0100',
        # Parte
        nome_parte='Michel Metran da Silva',
        nome_parte_pesquisa_nome_completo=True,
        # documento_parte='43.547.234-74',
        # Advogado
        # nome_advogado='Fernanda Dal Picolo',
        # nome_advogado_pesquisa_nome_completo=True,
        # oab='178.780',
        # Outros
        # n_carta_precatoria_origem='123.456.789',
        # n_documento_delegacia='001.431.473-67',
        # cda='01.432.326-0001/55',
    )
    print(proced)

    # Intimações
    pesquisa(driver=driver2, dados=proced)

    time.sleep(2)
    driver2.quit()



"""
_summary_

:raises Exception: _description_
:return: _description_
:rtype: _type_
"""

import platform
import tarfile
import tempfile
from pathlib import Path
from zipfile import ZipFile

import requests

from pyesaj.scraper.webdriver import config


class Gecko:
    """
    _summary_
    """

    def __init__(self) -> None:
        """
        _summary_

        :return: _description_
        :rtype: _type_
        """

        # Temp Path
        temp_path = tempfile.gettempdir()
        project_temp_path = Path(temp_path) / config.TEMP_PATH_NAME

        # Scrapy Path
        scrapy_path = project_temp_path / 'scrapy'
        scrapy_path.mkdir(exist_ok=True, parents=True)

        # Driver Path
        self.driver_path = scrapy_path / 'driver'
        self.driver_path.mkdir(exist_ok=True, parents=True)

        # Drivers por Sistema Operacional
        if platform.system() == 'Windows':
            gecko_win_filepath = self.driver_path / 'geckodriver.exe'
            if gecko_win_filepath.is_file():
                self.has_geckodriver = True

            elif not gecko_win_filepath.is_file():
                self.has_geckodriver = False

        elif platform.system() == 'Linux':
            gecko_linux_filepath = self.driver_path / 'geckodriver'
            if gecko_linux_filepath.is_file():
                self.has_geckodriver = True

            elif not gecko_linux_filepath.is_file():
                self.has_geckodriver = False

    def _get_geckodriver(self, verify_ssl: bool):
        """
        Faz o download do geckodriver!

        :return:
        """

        # print(gecko_zip_filepath)
        if not self.has_geckodriver:
            if platform.system() == 'Windows':
                # Download do geckodriver
                r = requests.get(
                    config.URL_GECKODRIVER_WINDOWS,
                    timeout=60,
                    verify=verify_ssl,
                )

                # Save
                name_gecko = Path(config.URL_GECKODRIVER_WINDOWS).name
                gecko_zip_filepath = self.driver_path / name_gecko
                with open(gecko_zip_filepath, 'wb') as f:
                    f.write(r.content)

                # Extract
                with ZipFile(gecko_zip_filepath, 'r') as zip_ref:
                    zip_ref.extractall(self.driver_path)

            elif platform.system() == 'Linux':
                # Download do geckodriver
                r = requests.get(
                    config.URL_GECKODRIVER_LINUX, timeout=60, verify=verify_ssl
                )

                # Save
                name_gecko = Path(config.URL_GECKODRIVER_LINUX).name
                gecko_zip_filepath = self.driver_path / name_gecko
                with open(gecko_zip_filepath, 'wb') as f:
                    f.write(r.content)

                # Extract
                with tarfile.open(gecko_zip_filepath, 'r') as tar_ref:
                    tar_ref.extractall(self.driver_path)

        elif self.has_geckodriver:
            print(f'Geckodriver already in {self.driver_path}')

    def get_path_geckodriver(self, verify_ssl: bool = False) -> Path | str:
        """
        dddd
        """

        # Faz download se for necessário
        self._get_geckodriver(verify_ssl=verify_ssl)

        # Path
        if platform.system() == 'Windows':
            _gecko_path = self.driver_path / 'geckodriver.exe'
            _gecko_path = _gecko_path.resolve().as_posix().replace('/', '\\')
            return _gecko_path

        elif platform.system() == 'Linux':
            _gecko_path = self.driver_path / 'geckodriver'
            return _gecko_path

        else:
            raise Exception(f'Ajustar para plataforma {platform.system()}')


if __name__ == '__main__':
    import time

    from pyesaj import webdriver

    # Instancia Driver
    driver = webdriver.Firefox(
        # driver_path=paths.driver_path,
        # logs_path=paths.app_logs_path,
        # down_path=paths.driver_path,
        headless=False,
        verify_ssl=False,
    )
    # Add xPath Extension
    driver.add_extension_xpath()
    driver.get(url='https://www.uol.com.br/')
    time.sleep(3)
    driver.quit()

    gecko = Gecko()
    gecko_path = gecko.get_path_geckodriver()
    print(gecko_path)



"""
Módulo para pesquisar
Intimações a serem recebidas
"""

import time
from typing import Union

from selenium import webdriver

import pyesaj as esaj
from pyesaj.scraper.page.intim import CheckError, CheckResults, Recebimento, \
    Consulta
from pyesaj.scraper.params.intim.input import ConsultaIntimacoes, \
    RecebeIntimacoes


def especializa(
    driver: webdriver.Firefox,
    dados: Union[RecebeIntimacoes, ConsultaIntimacoes],
) -> tuple[bool, CheckError, CheckResults]:
    """
    Pesquisa pelas intimações

    Retorna um bool se tem ou não tabela!
    """
    # Vai para página
    if isinstance(dados, RecebeIntimacoes):
        Recebimento(driver=driver)

    elif isinstance(dados, ConsultaIntimacoes):
        Consulta(driver=driver)

    else:
        raise Exception(f'Precisa ser Consulta ou Recebimento, é {type(dados)}')

    # Atributo: Em Nome De
    if dados.em_nome_de is not None:
        intim_em_nome = esaj.scraper.page.intim.EmNomeDe(driver=driver)
        intim_em_nome.set_option(option=dados.em_nome_de)

    # Atributo: Tipo de Participação
    if dados.tipo_participacao is not None:
        intim_tipo = esaj.scraper.page.intim.TipoParticipacao(driver=driver)
        intim_tipo.set_option(option=dados.tipo_participacao)

    # Atributo: Instância
    if dados.instancia is not None:
        intim_inst = esaj.scraper.page.intim.Instancia(driver=driver)
        intim_inst.set_option(instancia=dados.instancia)

    # Atributo: Foro
    if dados.foro is not None:
        intim_foro = esaj.scraper.page.intim.Foro(driver=driver)
        intim_foro.set_option(option=dados.foro)

    # Atributo: Vara
    if dados.vara is not None:
        intim_vara = esaj.scraper.page.intim.Vara(driver=driver)
        intim_vara.set_option(option=dados.vara)

    # Atributo: Seção
    if dados.secao is not None:
        intim_secao = esaj.scraper.page.intim.Secao(driver=driver)
        intim_secao.set_option(option=dados.secao)

    # Atributo: Órgão Julgador
    if dados.orgao_julgador is not None:
        intim_org = esaj.scraper.page.intim.OrgaoJulgador(driver=driver)
        intim_org.set_option(option=dados.orgao_julgador)

    # Atributo: Especialização
    if dados.especializacao is not None:
        intim_esp = esaj.scraper.page.intim.Especializacao(driver=driver)
        intim_esp.set_option(option=dados.especializacao)

    # Atributo: Especialização Checkbox
    if dados.especializacao_nao_definida is True:
        intim_esp = esaj.scraper.page.intim.Especializacao(driver=driver)
        intim_esp.apenas_processos_sem_especializacao()

    # Atributo: Cargo
    if dados.cargo is not None:
        intim_cargo = esaj.scraper.page.intim.Cargo(driver=driver)
        intim_cargo.set_option(option=dados.cargo)

    # Atributo: Cargo Checkbox
    if dados.especializacao_nao_definida is True:
        intim_cargo = esaj.scraper.page.intim.Cargo(driver=driver)
        intim_cargo.apenas_processos_sem_cargo()

    # Atributo: Classe
    if dados.classe is not None:
        intim_classe = esaj.scraper.page.intim.Classe(driver=driver)
        intim_classe.set_option(option=dados.classe)

    # Atributo: Assunto Principal
    if dados.assunto_principal is not None:
        intim_assunto = esaj.scraper.page.intim.AssuntoPrincipal(driver=driver)
        intim_assunto.set_option(option=dados.assunto_principal)

    # Atributo: Área
    if dados.area is not None:
        intim_area = esaj.scraper.page.intim.Area(driver=driver)
        intim_area.set_option(option=dados.area)

    # Atributo: Processo
    if dados.processo is not None:
        intim_proc = esaj.scraper.page.intim.Processo(driver=driver)
        intim_proc.write(texto=dados.processo)

    # Atributo: Natureza de Comunicação
    if dados.natureza_comunicacao is not None:
        intim_nat = esaj.scraper.page.intim.NaturezaComunicacao(driver=driver)
        intim_nat.set_option(natureza=dados.natureza_comunicacao)

    if isinstance(dados, ConsultaIntimacoes):

        # Período: De / Exclusivo do "Consulta"
        if dados.periodo_de is not None:
            intim_per = esaj.scraper.page.intim.consulta.Periodo(driver=driver)
            intim_per.de(data=dados.periodo_de)

        # Período: Até / Exclusivo do "Consulta"
        if dados.periodo_ate is not None:
            intim_per = esaj.scraper.page.intim.consulta.Periodo(driver=driver)
            intim_per.ate(data=dados.periodo_ate)

        # Período: Intervalo / Exclusivo do "Consulta"
        if dados.periodo_de is not None and dados.periodo_ate is not None:
            intim_per = esaj.scraper.page.intim.consulta.Periodo(driver=driver)
            intim_per.define_intervalo(
                de=dados.periodo_de, ate=dados.periodo_ate
            )

        # Atributo: Ciência do Ato / Exclusivo do "Consulta"
        if dados.ciencia_ato is not None:
            intim_cien = esaj.scraper.page.intim.consulta.CienciaAto(
                driver=driver)
            intim_cien.set_option(option=dados.ciencia_ato)

        # Atributo: Situação / Exclusivo do "Consulta"
        if dados.situacao is not None:
            intim_sit = esaj.scraper.page.intim.consulta.Situacao(driver=driver)
            intim_sit.set_option(situacao=dados.situacao)

    # Aguarda
    time.sleep(1)

    # Consulta
    consulta = esaj.scraper.page.intim.ConsultarIntimacoes(driver=driver)
    consulta.consultar()

    # Avalia se tem Resultados
    err = esaj.scraper.page.intim.CheckError(driver=driver)
    # erro = err.has_errors()
    # if erro:
    #     logging.warning(msg=f'Existem erros na consulta')

    res = esaj.scraper.page.intim.CheckResults(driver=driver)
    # tem_res = res.tem_resultado()
    return err, res

    # # Tem erro?
    # if not tem_res:
    #     # logging.info(msg=f'Não existem resultados')
    #     return False, err, res

    # else:
    #     chk_tab = consulta.check_table()
    #     # logging.info(msg=f'Tabel tabela {chk_tab}')
    #     return chk_tab, err, res


if __name__ == '__main__':
    import os

    from dotenv import load_dotenv

    # Credenciais
    load_dotenv()
    USERNAME = os.getenv('USERNAME_TJSP')
    PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver2 = esaj.scraper.webdriver.Firefox(verify_ssl=False)

    # Faz Login
    log = esaj.scraper.page.Login(driver=driver2)
    log.login(username=USERNAME, password=PASSWORD)

    # Define
    intim_search = RecebeIntimacoes(
        em_nome_de='Ministério Público do Estado de São Paulo',
        instancia='Segundo Grau',
        especializacao_nao_definida=True,
        cargo_nao_definido=True,
        area='Ambas',
        natureza_comunicacao='Ambas',
    )
    print(intim_search)

    # Intimações
    especializa(driver=driver2, dados=intim_search)

    time.sleep(2)
    driver2.quit()



"""
Módulo para usar driver do Chrome

Michel Metran
Data: 24.10.2024
Atualizado em: 24.10.2024
"""

import tempfile

from selenium import webdriver
from selenium.webdriver.chrome.options import Options as ChromeOptions
from pathlib import Path
from pyesaj.scraper.webdriver import config

# from selenium.webdriver.chrome.service import Service as ChromeService


class Chrome(webdriver.Chrome):
    """
    Cria driver customizado do Selenium

    :param webdriver: _description_
    :type webdriver: _type_
    """

    def __init__(
        self,
        # driver_path: Path,
        # logs_path: Path,
        # down_path: Path,
        *args,
        **kwargs,
    ):
        """
        - verify_ssl
        - headless
        - download_path
        """
        # Parameters
        headless = kwargs.get('headless', False)
        self.download_path = kwargs.get('download_path', False)

        # Temp Path
        temp_path = tempfile.gettempdir()
        project_temp_path = Path(temp_path) / config.TEMP_PATH_NAME

        # Scrapy Path
        scrapy_path = project_temp_path / 'scrapy'
        scrapy_path.mkdir(exist_ok=True, parents=True)

        # Download Path
        if self.download_path is False:
            # Cria Pasta
            self.download_path = scrapy_path / 'download'
            self.download_path.mkdir(exist_ok=True, parents=True)

        # my_service = ChromeService()
        # print(str(self.download_path))
        # print(self.download_path)
        # print(self.download_path.is_dir())
        # print(str(self.download_path) + os.path.sep)

        # Options
        options = ChromeOptions()
        options.add_argument('--start-maximized')
        options.add_argument('--disable-gpu')

        # Se tem Modo Anônimo, o download não funciona adequadamente
        # options.add_argument('--incognito')

        # Certificados
        options.add_argument('--ignore-certificate-errors-spki-list')
        options.add_argument('--ignore-certificate-errors')
        options.add_argument('--ignore-ssl-errors')

        options.add_argument('--disable-infobars')
        options.add_argument('--disable-extensions')
        # options.add_argument('--disable-logging')
        # Remove a mensagem "Chrome is being controlled by automated test software"
        # que aparece quando o Chrome é iniciado pelo Selenium.
        options.add_experimental_option(
            'excludeSwitches', ['enable-automation']
        )
        # Desativa a extensão de automação do Chrome que é carregada por
        # padrão quando o Chrome é iniciado pelo Selenium.
        # Isso ajuda a evitar que sites detectem que o navegador está
        # sendo controlado por um script de automação.
        options.add_experimental_option('useAutomationExtension', False)

        options.add_experimental_option(
            'prefs',
            {
                'credentials_enable_service': False,
                'profile.password_manager_enabled': False,
                'download.default_directory': str(self.download_path),
                # + os.path.sep,
                'download.prompt_for_download': False,
                'download.directory_upgrade': True,
                'safebrowsing.enabled': True,
            },
        )

        if headless is True:
            options.add_argument('--headless')

        super().__init__(
            # service=my_service,
            options=options
        )
        self.maximize_window()


if __name__ == '__main__':
    from pathlib import Path

    from selenium.webdriver.common.by import By

    import pyesaj as esaj

    # # Credenciais
    # load_dotenv()
    # USERNAME = os.getenv('USERNAME_TJSP')
    # PASSWORD = os.getenv('PASSWORD_TJSP')
    # Instancia Driver
    driver = esaj.webdriver.Chrome(headless=False)

    driver.get(url='https://filesamples.com/formats/csv')

    print('vai pra botão')
    btn = driver.find_element(
        By.XPATH, '//a[@href="/samples/document/csv/sample4.csv"]'
    )
    print('clica!')
    btn.click()

    # # Login
    # log = esaj.page.Login(driver=driver)
    # log.login(username=USERNAME, password=PASSWORD)

    # # Intimações
    # processo = '2336412-80.2024.8.26.0000'

    # # Define
    # intim_search = esaj.params.intim.input.ConsultaIntimacoes(
    #     em_nome_de='Ministério Público do Estado de São Paulo',
    #     instancia='Segundo Grau',
    #     # secao='Direito Criminal',
    #     # orgao_julgador='17ª Câmara de Direito Privado B',
    #     # especializacao='Criminal',
    #     # especializacao_nao_definida=True,
    #     # cargo='Secretaria ProcCriminal',
    #     # cargo_nao_definido=True,
    #     # assunto_principal='10527 - Livros / Jornais / Periódicos',
    #     area='Ambas',
    #     ciencia_ato='Todos',
    #     natureza_comunicacao='Ambas',
    #     situacao='Ambas',
    #     processo=processo,
    # )

    # dados = intim_search
    # print(intim_search)

    # # Vai para página
    # esaj.page.intim.Consulta(driver=driver)

    # # Atributo: Em Nome De
    # if dados.em_nome_de is not None:
    #     intim_em_nome = esaj.page.intim.EmNomeDe(driver=driver)
    #     intim_em_nome.set_option(option=dados.em_nome_de)

    # # Atributo: Tipo de Participação
    # if dados.tipo_participacao is not None:
    #     intim_tipo = esaj.page.intim.TipoParticipacao(driver=driver)
    #     intim_tipo.set_option(option=dados.tipo_participacao)

    # # Atributo: Instância
    # if dados.instancia is not None:
    #     intim_inst = esaj.page.intim.Instancia(driver=driver)
    #     intim_inst.set_option(instancia=dados.instancia)

    # # Atributo: Foro
    # if dados.foro is not None:
    #     intim_foro = esaj.page.intim.Foro(driver=driver)
    #     intim_foro.set_option(option=dados.foro)

    # # Atributo: Vara
    # if dados.vara is not None:
    #     intim_vara = esaj.page.intim.Vara(driver=driver)
    #     intim_vara.set_option(option=dados.vara)

    # # Atributo: Seção
    # if dados.secao is not None:
    #     intim_secao = esaj.page.intim.Secao(driver=driver)
    #     intim_secao.set_option(option=dados.secao)

    # # Atributo: Órgão Julgador
    # if dados.orgao_julgador is not None:
    #     intim_org = esaj.page.intim.OrgaoJulgador(driver=driver)
    #     intim_org.set_option(option=dados.orgao_julgador)

    # # Atributo: Especialização
    # if dados.especializacao is not None:
    #     intim_esp = esaj.page.intim.Especializacao(driver=driver)
    #     intim_esp.set_option(option=dados.especializacao)

    # # Atributo: Especialização Checkbox
    # if dados.especializacao_nao_definida is True:
    #     intim_esp = esaj.page.intim.Especializacao(driver=driver)
    #     intim_esp.apenas_processos_sem_especializacao()

    # # Atributo: Cargo
    # if dados.cargo is not None:
    #     intim_cargo = esaj.page.intim.Cargo(driver=driver)
    #     intim_cargo.set_option(option=dados.cargo)

    # # Atributo: Cargo Checkbox
    # if dados.especializacao_nao_definida is True:
    #     intim_cargo = esaj.page.intim.Cargo(driver=driver)
    #     intim_cargo.apenas_processos_sem_cargo()

    # # Atributo: Classe
    # if dados.classe is not None:
    #     intim_classe = esaj.page.intim.Classe(driver=driver)
    #     intim_classe.set_option(option=dados.classe)

    # # Atributo: Assunto Principal
    # if dados.assunto_principal is not None:
    #     intim_assunto = esaj.page.intim.AssuntoPrincipal(driver=driver)
    #     intim_assunto.set_option(option=dados.assunto_principal)

    # # Atributo: Área
    # if dados.area is not None:
    #     intim_area = esaj.page.intim.Area(driver=driver)
    #     intim_area.set_option(option=dados.area)

    # # Período: De
    # if dados.periodo_de is not None:
    #     intim_per = esaj.page.intim.consulta.Periodo(driver=driver)
    #     intim_per.de(data=dados.periodo_de)

    # # Período: Até
    # if dados.periodo_ate is not None:
    #     intim_per = esaj.page.intim.consulta.Periodo(driver=driver)
    #     intim_per.ate(data=dados.periodo_ate)

    # # Período: Intervalo
    # if dados.periodo_de is not None and dados.periodo_ate is not None:
    #     intim_per = esaj.page.intim.consulta.Periodo(driver=driver)
    #     intim_per.define_intervalo(de=dados.periodo_de, ate=dados.periodo_ate)

    # # Atributo: Processo
    # if dados.processo is not None:
    #     intim_proc = esaj.page.intim.Processo(driver=driver)
    #     intim_proc.write(texto=dados.processo)

    # # Atributo: Ciência do Ato
    # if dados.ciencia_ato is not None:
    #     intim_cien = esaj.page.intim.consulta.CienciaAto(driver=driver)
    #     intim_cien.set_option(option=dados.ciencia_ato)

    # # Atributo: Natureza de Comunicação
    # if dados.natureza_comunicacao is not None:
    #     intim_nat = esaj.page.intim.NaturezaComunicacao(driver=driver)
    #     intim_nat.set_option(natureza=dados.natureza_comunicacao)

    # # Atributo: Situação
    # if dados.situacao is not None:
    #     intim_sit = esaj.page.intim.consulta.Situacao(driver=driver)
    #     intim_sit.set_option(situacao=dados.situacao)

    # # Aguarda
    # time.sleep(2)

    # # Consulta
    # consulta = esaj.page.intim.ConsultarIntimacoes(driver=driver)
    # consulta.consultar()

    # # Tabela
    # tab = esaj.page.intim.Tabela(driver=driver)
    # tab.get_table(
    #     # instancia=instancia,
    #     seleciona_processos=[processo],
    # )

    # acao = esaj.page.intim.Acoes(driver=driver)
    # csv_filepath = acao.export_csv()
    # print(csv_filepath)

    # driver.quit()



"""
Módulo para definição de objetos
que serão os parâmetros de input
das pesquisas de processos
"""

import warnings
from typing import Literal, Optional

from pydantic import BaseModel, model_validator


class PesquisaProcessos(BaseModel):
    """
    _summary_

    :param BaseModel: _description_
    :type BaseModel: _type_
    :raises ValueError: _description_
    :raises Exception: _description_
    :return: _description_
    :rtype: _type_
    """

    instancia: Literal['Primeiro Grau', 'Segundo Grau']
    consultar_por: Literal[
        'Número do Processo',
        'Nome da parte',
        'Documento da Parte',
        'Nome do Advogado',
        'OAB',
        'Nº da Carta Precatória na Origem',
        'Nº do Documento na Delegacia',
        'CDA',
    ]
    numero_unificado: Optional[str] = None
    nome_parte: Optional[str] = None
    nome_parte_pesquisa_nome_completo: Optional[bool] = None
    documento_parte: Optional[str] = None
    nome_advogado: Optional[str] = None
    nome_advogado_pesquisa_nome_completo: Optional[bool] = None
    oab: Optional[str] = None
    n_carta_precatoria_origem: Optional[str] = None
    n_documento_delegacia: Optional[str] = None
    cda: Optional[str] = None

    @model_validator(mode='after')
    def check_instancia(self):
        """
        A pesquisa de processos de 2º grau não permite a 'Consulta Por'
        alguns tipos autorizados nas pesquisas de processos de 1º grau.

        Portanto, faz-se necessária a validação.
        """
        consulta_por_2_grau = [
            'Número do Processo',
            'Nome da parte',
            'Documento da Parte',
            'Nome do Advogado',
            'OAB',
            # 'Nº da Carta Precatória na Origem',
            # 'Nº do Documento na Delegacia',
            # 'CDA'
        ]
        if self.instancia == 'Segundo Grau':
            if self.consultar_por not in consulta_por_2_grau:
                list_str = '\n'.join([f'- {x}' for x in consulta_por_2_grau])
                raise ValueError(
                    f'\n\n'
                    f'Para processos de {self.instancia} a pesquisa só pode ser feita por:\n'
                    f'{list_str}'
                    f'\n\n'
                )
        return self

    @model_validator(mode='after')
    def set_none(self, v):
        """
        Define "None" para as variáveis a depender do parâmetro "consultar_por",
        Mantendo apenas os parâmetros necessários àquela "consulta por".
        """

        dc_consulta_por_depende_de = {
            # Consulta Por: Depende De
            'Número do Processo': [{'nome': 'numero_unificado', 'tipo': str}],
            'Nome da parte': [
                {'nome': 'nome_parte'},
                {'nome': 'nome_parte_pesquisa_nome_completo', 'tipo': bool},
            ],
            'Documento da Parte': [{'nome': 'documento_parte', 'tipo': str}],
            'Nome do Advogado': [
                {'nome': 'nome_advogado', 'tipo': str},
                {'nome': 'nome_advogado_pesquisa_nome_completo', 'tipo': bool},
            ],
            'OAB': [{'nome': 'oab', 'tipo': str}],
            'Nº da Carta Precatória na Origem': [
                {'nome': 'n_carta_precatoria_origem', 'tipo': str}
            ],
            'Nº do Documento na Delegacia': [
                {'nome': 'n_documento_delegacia', 'tipo': str}
            ],
            'CDA': [{'nome': 'cda', 'tipo': str}],
        }

        list_depende_de_atrrib = dc_consulta_por_depende_de[self.consultar_por]
        list_depende_de = [x['nome'] for x in list_depende_de_atrrib]
        # print('*' * 100, list_depende_de)

        # Lista das Variáveis, exceto àquelas genéricas, que não serão definidas como None
        list_vars = [
            k
            for k, v in self.model_fields.items()
            if k not in ['instancia', 'consultar_por']
        ]
        # print('*' * 100, list_vars)

        for var in list_vars:
            if var in list_depende_de and eval(f'self.{var}') is None:
                raise Exception(f'A variável "{var}" precisa ser definida!')

            elif var not in list_depende_de:
                if eval(f'self.{var}') is not None:
                    # Mensagem
                    warnings.warn(
                        message=f'\n'
                        f'Na pesquisa por "{self.consultar_por}" depende dos atributos {list_depende_de}.\n'
                        f'O atributo "{var}" está como "{eval(f"self.{var}")}" e será definido como "None".'
                        f'\n',
                        category=RuntimeWarning,
                    )
                    # Set None
                    setattr(self, var, None)

        return self


if __name__ == '__main__':
    proced = PesquisaProcessos(
        instancia='Segundo Grau',
        # consultar_por='Número do Processo',
        consultar_por='Nome da parte',
        # consultar_por='CDA',
        # numero_unificado='0123479-07.2012.8.26.0100',
        # Parte
        nome_parte='Michel Metran',
        nome_parte_pesquisa_nome_completo=False,
        # documento_parte='43.547.234-74',
        # Advogado
        # nome_advogado='Fernanda Dal Picolo',
        # nome_advogado_pesquisa_nome_completo=True,
        # oab='178.780',
        # Outros
        # n_carta_precatoria_origem='123.456.789',
        # n_documento_delegacia='001.431.473-67',
        # cda='01.432.326-0001/55',
    )
    print(proced)



"""
Módulo com variáveis sobre a definição do driver
E extensões do driver.
"""

# Nome da pasta que ficará dentro da pasta temp.
# Nessa pasta serão armazenados logs e drivers do Selenium
TEMP_PATH_NAME = 'app_procuradoria'

# Url do GeckoDriver
URL_GECKODRIVER_WINDOWS = 'https://github.com/mozilla/geckodriver/releases/download/v0.32.0/geckodriver-v0.32.0-win64.zip'
URL_GECKODRIVER_LINUX = 'https://github.com/mozilla/geckodriver/releases/download/v0.35.0/geckodriver-v0.35.0-linux64.tar.gz'

# URLs da Extensão xPath
URL_ADDONS_XPATH = 'https://addons.mozilla.org/firefox/downloads/file/3588871/xpath_finder-1.0.2-fx.xpi'



"""
Módulo com a página base do projeto
utilizando a estrutura de Page Object Model (POM)
"""

import logging
from abc import ABC
from typing import List, Optional, Union

from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import Select, WebDriverWait


class SeleniumObject:
    """
    Guarda os métodos do Selenium
    """

    def __init__(self, driver) -> None:
        self.driver = driver

    def go_to(self, url) -> None:
        """
        Abre a página web definida na variável `url`.
        Avalia se o `driver` já está na página solicitada e,
        em caso negativo, acessa a página.
        """
        if self.driver.current_url != url:
            self.driver.get(url)

        # Avalia se entrou na página e ela carregou completamente
        page_state = self.driver.execute_script('return document.readyState;')
        n_try = 0
        while page_state != 'complete' and n_try < 60 * 2:
            time.sleep(1)
            page_state = self.driver.execute_script(
                'return document.readyState;'
            )
            n_try += 1

    def find(self, locator: tuple, wait: float = 60) -> WebElement:
        """
        Encontra algum
        """
        if len(locator) == 2:
            start_point = self.driver

        elif len(locator) == 3:
            start_point = locator[2]
            locator = locator[0:2]

        else:
            raise ValueError('Erro no input da tupla')

        try:
            if wait != 0:
                elem = WebDriverWait(start_point, wait).until(
                    EC.presence_of_element_located((locator))
                )
            else:
                # Asterisco serve para desempacotar os parâmetros de input
                elem = start_point.find_element(*locator)

        except Exception as e:
            logging.error(
                f'e-SAJ parece não ter carregado corretamente os elemento!. Veja erro:\n{e}'
            )

        return elem

    def find_all(self, locator: tuple, wait: int = 60) -> List[WebElement]:
        """
        Encontra todos
        """
        if len(locator) == 2:
            start_point = self.driver

        elif len(locator) == 3:
            start_point = locator[2]
            locator = locator[0:2]

        else:
            raise ValueError('Erro no input da tupla')

        try:
            if wait != 0:
                elem = WebDriverWait(start_point, wait).until(
                    EC.presence_of_all_elements_located((locator))
                )
            else:
                # Asterisco serve para desempacotar os parâmetros de input
                elem = start_point.find_elements(*locator)

        except Exception as e:
            logging.error(
                f'e-SAJ parece não ter carregado corretamente os elemento!. Veja erro:\n{e}'
            )

        return elem

    def set(self, locator, value) -> None:
        """
        Define algum campo para preenchimento

        Vi o conceito de aguardar em:
        https://stackoverflow.com/questions/73169697/how-to-wait-for-text-value-to-show-using-selenium-python
        """
        self.find(locator).clear()
        self.find(locator).send_keys(value)

        WebDriverWait(self.driver, 10).until(
            EC.text_to_be_present_in_element_value(locator, text_=value)
        )

    def click(self, locator) -> None:
        """
        Clica em algum botão
        """
        self.find(locator).click()

    def get_text(self, locator, *args, **kwargs) -> str:
        """
        Pego text
        """
        wait = kwargs.get('wait', 60)
        try:
            return self.find(locator, wait).text.strip()

        except:
            return None

    def attribute(self, locator, attribute):
        """
        Pega atributo
        """
        return self.find(locator).get_attribute(attribute)

    def create_select_options(
        self,
        locator,
        options_delete: Union[Optional[str], Optional[List[str]]] = None,
    ) -> List[str]:
        """
        Cria uma lista das opções em forma de texto
        É possível passar parâmetro opções para excluir itens da lista
        """
        try:
            selector = Select(self.find(locator))
            list_options = [x.text for x in selector.options]

        except Exception as e:
            logging.error(f'e-SAJ não carregou opções. Erro  {e}')

        if options_delete is None:
            options_delete = ['-- Selecione --', '']

        elif isinstance(options_delete, list):
            options_delete.extend(['-- Selecione --', ''])

        list_options = list(
            set([x for x in list_options if x not in options_delete])
        )
        list_options.sort()
        return list_options

    def select(self, locator, option) -> None:
        """
        Seleciona uma opção
        """
        selector = Select(self.find(locator))
        selector.select_by_visible_text(option)

    def switch(self, locator) -> None:
        """
        _summary_
        """
        iframe_xpath = self.find(locator)
        self.driver.switch_to.frame(iframe_xpath)

    def switch_back(self) -> None:
        """
        _summary_
        """
        self.driver.switch_to.default_content()

    def zoom(self, zoom):
        self.driver.execute_script(f"document.body.style.zoom='{zoom}%'")

    def script(self, script, *arg):
        self.driver.execute_script(script, *arg)


class Page(ABC, SeleniumObject):
    """
    Modelo Padrão de Página
    """

    def __init__(self, driver):
        super().__init__(driver)
        # self.driver = driver
        # self._reflection()

    # def _reflection(self):
    #     """
    #     Define qual o driver nos PageElements
    #     """
    #     for atributo in dir(self):
    #         atributo_real = getattr(self, atributo)
    #         print(atributo, atributo_real)
    #         if isinstance(atributo_real, PageElement):
    #             atributo_real.driver = self.driver


class PageElement(ABC, SeleniumObject):
    """
    Modelo Padrão dos Elementos das Páginas
    """

    def __init__(self, driver):
        super().__init__(driver)
        # self.driver = driver


if __name__ == '__main__':
    import time

    from selenium.webdriver.common.by import By

    from pyesaj import webdriver

    #
    my_driver = webdriver.Firefox(
        # driver_path=paths.driver_path,
        # logs_path=paths.app_logs_path,
        # down_path=paths.driver_path,
        verify_ssl=False,
    )

    my_driver.get('https://www.youtube.com/watch?v=KM90nnkt-5w')
    print('-' * 100)

    #
    page = Page(driver=my_driver)
    page.go_to('https://www.youtube.com/')

    page.find(locator=(By.XPATH, '//div'))

    time.sleep(2)
    my_driver.quit()



"""
Módulo com a página de cadastro do eSAJ
Para pegar o que o usuário tem autorização para fazer
"""

from typing import List

from selenium.webdriver.common.by import By

from pyesaj.scraper.params import perfil
from pyesaj.scraper.pom import Page


# flake8: noqa:E501


class GerenciamentoPerfis(Page):
    """
    Página "Cadastro > Gerenciamento de Perfis"
    """

    url = 'https://esaj.tjsp.jus.br/esajperfil/abrirGerenciamentoDePerfis.do'
    perfis = (By.XPATH, '//div[@class="titPerfil"]')

    def __init__(self, driver) -> None:
        super().__init__(driver)
        # Vai para a página
        self.go_to(self.url)

    def obtem_perfis(self) -> List[perfil.Perfil]:
        """
        Obtém perfis de acesso do usuário logado.
        """

        # Itera pelos Perfis
        list_perfis = []
        for item in self.find_all(locator=self.perfis):
            # Define Variáveis
            # Precisei definir aqui pois tem o "item" que é variável
            perfis_atrib = (By.XPATH, './input', item)
            perfis_description = (
                By.XPATH,
                './../div[@class="contPerfil" and @style="display: block;"]',
                item,
            )
            open_profile = (By.XPATH, './/img[@class="setaCima"]', item)
            close_profile = (
                By.XPATH,
                './/img[@class="setaBaixo" and @style="display: inline;"]',
                item,
            )

            # Profile Name
            print(item.text)

            # Pega atributo: se está habilitado
            authorization = self.attribute(
                locator=perfis_atrib, attribute='checked'
            )
            if authorization == 'true':
                authorization_fix = True

            else:
                authorization_fix = False

            # Abre Perfil
            self.click(open_profile)

            # Lê conteúdo: Descrição e Observação
            msg = self.get_text(perfis_description)

            # Ajusta Descrição
            msgs = msg.split('\n', maxsplit=1)
            if isinstance(msgs, list) and len(msgs) > 1:
                desc = msgs[0].strip().strip('\n')
                obs = msgs[1].strip().strip('\n')

            else:
                desc = msg.strip().strip('\n')
                obs = None

            # Fecha Perfil
            self.click(close_profile)

            # Apensa Perfil
            list_perfis.append(
                perfil.Perfil(
                    perfil=item.text,
                    autorizado=authorization_fix,
                    descricao=desc,
                    observacao=obs,
                )
            )

        return list_perfis


class DadosCadastrais(Page):
    """
    Página "Cadastro > Dados Cadastrais"
    """

    url = 'https://esaj.tjsp.jus.br/esajperfil/abrirEdicaoDeDadosDoUsuario.do'
    nome = (By.XPATH, '//input[@id="identity.nmUsuario"]')
    nome_social = (By.XPATH, '//input[@id="identity.nmSocial"]')
    email = (By.XPATH, '//input[@id="identity.deEmail"]')
    email_altenativo = (By.XPATH, '//input[@id="identity.deEmailAlternativo"]')
    cpf = (By.XPATH, '//input[@id="identity.nuCpfCnpj"]')
    rg = (By.XPATH, '//input[@id="identity.nuRg"]')
    rg_orgao_emissor = (By.XPATH, '//input[@id="identity.deOrgaoemissor"]')
    telefone = (By.XPATH, '//input[@id="identity.nuTelefone"]')
    celular = (By.XPATH, '//input[@id="identity.nuCelular"]')
    sexo_masculino = (
        By.XPATH,
        '//input[@id="id_esajperfil.label.usuario.tpGenero.masculino"]',
    )
    sexo_feminino = (
        By.XPATH,
        '//input[@id="id_esajperfil.label.usuario.tpGenero.feminino"]',
    )

    def __init__(self, driver) -> None:
        super().__init__(driver)
        self.go_to(url=self.url)

    def obtem_dados(self) -> perfil.Cadastro:
        """
        Obtém dados cadastrais do usuário logado.
        """

        # Sexo
        masc = self.attribute(locator=self.sexo_masculino, attribute='checked')
        fem = self.attribute(locator=self.sexo_feminino, attribute='checked')
        if masc == 'true' and fem is None:
            sexo = 'Masculino'

        elif masc is None and fem == 'true':
            sexo = 'Feminino'

        else:
            sexo = None

        return perfil.Cadastro(
            nome=self.attribute(self.nome, attribute='value'),
            nome_social=self.attribute(self.nome_social, attribute='value'),
            email=self.attribute(self.email, attribute='value'),
            email_alternativo=self.attribute(
                self.email_altenativo, attribute='value'
            ),
            cpf=self.attribute(self.cpf, attribute='value'),
            rg=self.attribute(self.rg, attribute='value'),
            rg_orgao_emissor=self.attribute(
                self.rg_orgao_emissor, attribute='value'
            ),
            telefone=self.attribute(self.telefone, attribute='value'),
            celular=self.attribute(self.celular, attribute='value'),
            genero=sexo,
        )


if __name__ == '__main__':
    import os
    import time

    from dotenv import load_dotenv

    import pyesaj as esaj

    # Credenciais
    load_dotenv()
    USERNAME = os.getenv('USERNAME_TJSP')
    PASSWORD = os.getenv('PASSWORD_TJSP')

    # Cria Driver
    driver_ff = esaj.scraper.webdriver.Firefox(verify_ssl=False)

    # Entra no eSAJ e loga
    log = esaj.scraper.page.Login(driver=driver_ff)
    log.login(username=USERNAME, password=PASSWORD)

    # Cadastro
    esaj_perfil = esaj.scraper.page.GerenciamentoPerfis(driver=driver_ff)
    perfis = esaj_perfil.obtem_perfis()
    print(perfis)

    dados_cadastrais = esaj.scraper.page.DadosCadastrais(driver=driver_ff)
    dados = dados_cadastrais.obtem_dados()
    print(dados)

    time.sleep(5)
    driver_ff.close()



"""
Módulo para funções diversas
"""

import logging


def format_number(n_esaj):
    """
    Notei que alguns números podem não estar adequadamente configurados.
    A Procuradoria da Câmara Especial trabalha dessa maneira.

    O Padrão do eSAJ tem 25 números.

    Converte
    De:   1028803-47.2022.8.26.0602/50000
    Para: 1028803-47.2022.8.26.0602
    """
    number_esaj_splitted = n_esaj.strip().split('/')
    n_esaj = number_esaj_splitted[0]

    if len(n_esaj) != 25:
        # print(f'O número tem {len(n_esaj)} caracteres')
        logging.info(
            f'O número do processo não tem 25 caracteres (padrão eSAJ). Tem {len(n_esaj)} caracteres!'
        )

    return n_esaj


if __name__ == '__main__':
    a = format_number(n_esaj='1028803-47.2022.8.26.0602/50000')
    print(a)

    a = format_number(n_esaj='1028803-47.2022.8.26.0602')
    print(a)

    print('8.26' in a)

    # import pyesaj as esaj
    # esaj.outros.format.format_number(n_esaj=)



"""
Módulo com a página de
"Definir especialização/cargo"
"""

import time
from urllib.parse import urlparse

from selenium.webdriver.common.by import By

from pyesaj.scraper.page.components.components import InputModelSearchSet
from pyesaj.scraper.pom import PageElement

# flake8: noqa:303
# flake8: noqa:E501


# class Define(Page):
#     """
#     Página "Intimações On-line > Definir especialização/cargo"
#     """

#     url = 'https://esaj.tjsp.jus.br/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosNaoRecebidosSelecionados.do'

#     def __init__(self, driver):
#         super().__init__(driver)
#         self.go_to(self.url)


class Especializacao(InputModelSearchSet):
    """
    Representa o campo de *input* "Especialização", para definir
    qual a Especialização que a intimação receberá.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td/input[@id="especialidade.nmEspecialidade"]/../following-sibling::td//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="nmEspecialidade"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.nmEspecialidade"]')
        self.lista_opcoes = None

        # Confere se está na página Correta
        path = urlparse(url=self.driver.current_url).path
        if path not in [
            '/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosRecebidosSelecionados.do',
            '/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosNaoRecebidosSelecionados.do',
            # Chega nessa situação/url após dar dois cliques em "Confirmar". 26.11.2024
            '/intimacoesweb/cadastrarEspecialidade.do',
        ]:
            raise Exception(
                'Necessário estar na página "Definir especialização/cargo"'
            )


class Cargo(InputModelSearchSet):
    """
    Representa o campo de *input* "Cargo", para definir
    qual o Cargo que a intimação receberá.
    """

    def __init__(self, driver):
        super().__init__(driver)

        self.abre_consulta = (
            By.XPATH,
            '//td/input[@id="cargo.deCargo"]/../following-sibling::td//img[@title="Abre a consulta"]',
        )
        self.linhas_tabela = (
            By.XPATH,
            '//table[@id="tabelaResultado"]//td[@property="deCargo"]',
        )
        self.input_opcao = (By.XPATH, '//input[@id="entity.deCargo"]')
        self.lista_opcoes = None

        # Confere se está na página Correta
        path = urlparse(url=self.driver.current_url).path
        if path not in [
            '/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosRecebidosSelecionados.do',
            '/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosNaoRecebidosSelecionados.do',
        ]:
            raise Exception(
                'Necessário estar na página "Definir especialização/cargo"'
            )


class Acoes(PageElement):
    """
    Represante os botões que promovem "Ações", tais como:
    - Confirmar
    - Voltar
    """

    btn_confirmar = (
        By.XPATH,
        '//table[@class="secaoBotoesBody"]//input[@id="definirEspecialidade" and @value="Confirmar"]',
    )
    btn_voltar = (
        By.XPATH,
        '//table[@class="secaoBotoesBody"]//input[@id="voltar"]',
    )

    def confirmar(self) -> None:
        """
        Aperta botão "Confirmar" para definir especialização e cargo indicados.
        """
        self.click(self.btn_confirmar)

    # def voltar(self) -> None:
    #     """
    #     Aperta botão "Voltar".
    #     """
    #     # TODO: Enquanto não chegar na página de volta
    #     path = urlparse(url=self.driver.current_url).path
    #     url_que_desejo_chegar = path in [
    #         '/intimacoesweb/consultarAtosNaoRecebidos.do',
    #         # TODO: Confirmar
    #         # Chute que é essa URL do menu "Consulta"
    #         '/intimacoesweb/consultarAtosRecebidos.do',
    #     ]
    #     # Clica
    #     self.click(self.btn_voltar)

    #     # Aguarda
    #     time.sleep(0.5)

    #     i = 0
    #     while url_que_desejo_chegar is not True and i <= 10:
    #         try:
    #             # Clica pra voltar novamente
    #             self.click(self.btn_voltar)
    #         except:
    #             pass

    #         # Aguarda
    #         time.sleep(0.5)

    #         # Recheck
    #         path = urlparse(url=self.driver.current_url).path
    #         url_que_desejo_chegar = path in [
    #             '/intimacoesweb/consultarAtosNaoRecebidos.do',
    #             # TODO: Confirmar
    #             # Chute que é essa URL do menu "Consulta"
    #             '/intimacoesweb/consultarAtosRecebidos.do',
    #         ]
    #         i += 1

    def voltar(self) -> None:
        """
        Aperta botão "Voltar".
        """

        def url_desejada(path) -> bool:
            """
            Avalia se estou na url que desejo ao apertar o botão voltar
            """
            return path in [
                '/intimacoesweb/consultarAtosNaoRecebidos.do',
                '/intimacoesweb/consultarAtosRecebidos.do',
            ]

        # Clica
        self.click(self.btn_voltar)
        time.sleep(0.5)

        i = 0
        while (
            not url_desejada(urlparse(url=self.driver.current_url).path)
            and i <= 10
        ):
            try:
                self.click(self.btn_voltar)
            except Exception as e:
                print(f"Erro ao clicar no botão voltar: {e}")

            time.sleep(0.5)
            i += 1


class Mensagem(PageElement):
    """
    Representa a mensagem após especializar,
    ou seja, após apertar o botão "confirmar".
    """

    msg = (
        By.XPATH,
        '//div[@id="spwTabelaMensagem"]//td[@id="mensagemRetorno"]',
    )

    def __init__(self, driver):
        super().__init__(driver)

        # Confere se está na página Correta
        path = urlparse(url=self.driver.current_url).path
        if path not in [
            '/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosRecebidosSelecionados.do',
            '/intimacoesweb/abrirTelaDeDefinicaoEspecialidadeParaAtosNaoRecebidosSelecionados.do',
        ]:
            raise Exception(
                f'Necessário estar na página "Definir especialização/cargo"\nEstou na url\n{path}'
            )

    def get_msg(self) -> str:
        """
        Obtém a mensagem após especializar
        Caso não encontre banner, retornará "False".
        """
        try:
            return self.get_text(locator=self.msg)

        except:
            # Não encontrou sequer o banner
            return False

    def sucesso(self) -> bool:
        """
        Avalia se a especialização foi feita com sucesso.
        """
        return self.get_msg() == 'Operação realizada com sucesso'


if __name__ == '__main__':
    pass
